@@ -1,11 +1,40 @@
+#' Installing Courses
+#' 
+#' swirl is designed so that anyone can create interactive content
+#' and share it with the world or with just a few people. Users can
+#' install courses from a variety of sources using the 
+#' functions listed here. Each of these functions has its own help
+#' file, which you can consult for more details.
+#' 
+#' If you're just getting started, we recommend using 
+#' \code{\link{install_from_swirl}} to install courses
+#' from our official \href{https://github.com/swirldev/swirl_courses}{course repository}. Otherwise, check out the
+#' help file for the relevant install function below.
+#' 
+#' You can uninstall a course from swirl at any time with 
+#' \code{\link{uninstall_course}}.
+#' 
+#' @name InstallCourses
+#' @family InstallCourses
+NULL
+
+
 #' Install a course from the official course repository
 #' 
+#' @description
 #' We are currently maintaining a central repository of contributed
 #' swirl courses at \url{https://github.com/swirldev/swirl_courses}.
 #' This function provides the easiest method of installing a course
 #' form the repository.
 #' 
+#' We have another repository at 
+#' \url{https://github.com/swirldev/swirl_misc}, where we keep 
+#' experimental features and content. The \code{dev} argument allows
+#' you to access this repository. Content in the swirl_misc repository
+#' is not guaranteed to work.
+#' 
 #' @param course_name The name of the course you wish to install.
+#' @param dev Set to \code{TRUE} to install a course in development from the swirl_misc repository.
 #' @export
 #' @importFrom httr GET content
 #' @examples
@@ -16,13 +45,29 @@
 #' ### OR ###
 #' 
 #' install_from_swirl("R Programming") # Course name
+#' 
+#' # To install a course in development from the swirl_misc repository
+#' install_from_swirl("Including Data", dev = TRUE)
 #' }
-install_from_swirl <- function(course_name){
+#' @family InstallCourses
+install_from_swirl <- function(course_name, dev = FALSE){
+  # Validate arguments
+  if(!is.character(course_name)) {
+    stop("Argument 'course_name' must be surrounded by quotes (i.e. a character string)!")
+  }
+  if(!is.logical(dev)) {
+    stop("Argument 'dev' must be either TRUE or FALSE!")
+  }
+  
   # make pathname from course_name
   course_name <- make_pathname(course_name)
   
-  # Construct url to the zip file
-  url <- "http://github.com/swirldev/swirl_courses/zipball/master"
+  # Construct url to the appropriate zip file
+  if(dev) {
+    url <- "http://github.com/swirldev/swirl_misc/zipball/master"
+  } else {
+    url <- "http://github.com/swirldev/swirl_courses/zipball/master"
+  }
   
   # Send GET request
   response <- GET(url)
@@ -87,6 +132,7 @@ install_from_swirl <- function(course_name){
 #' zip_course("~/Desktop/LOESS_Modeling")
 #' zip_course("~/Desktop/SNA_Tutorial", "~/tutorials")
 #' }
+#' @family InstallCourses
 zip_course <- function(path, dest=NULL){
   # Cleanse the path of the trailing slash
   path <- sub("/$", "", path)
@@ -136,6 +182,7 @@ zip_course <- function(path, dest=NULL){
 #' 
 #' uninstall_course("Linear Regression")
 #' }
+#' @family InstallCourses
 uninstall_course <- function(course_name){
   path <- file.path(system.file(package = "swirl"), "Courses", 
                     make_pathname(course_name))
@@ -162,6 +209,7 @@ uninstall_course <- function(course_name){
 #' install_course_zip("~/Downloads/swirl_courses-master.zip", multi=TRUE,
 #'                    which_course=c("R Programming", "Data Analysis"))
 #' }
+#' @family InstallCourses
 install_course_zip <- function(path, multi=FALSE, which_course=NULL){
   if(!is.logical(multi) || is.na(multi)) {
     stop("Argument 'multi' must be either TRUE or FALSE.")
@@ -224,6 +272,7 @@ install_course_zip <- function(path, multi=FALSE, which_course=NULL){
 #' 
 #' install_course_directory("~/Desktop/my_course")
 #' }
+#' @family InstallCourses
 install_course_directory <- function(path){
   # Check for size of directory to prevent copying a huge directory into swirl/Courses
   garbage_result <- tryCatch(
@@ -260,6 +309,7 @@ install_course_directory <- function(path){
 #' install_course_github("bcaffo", "Linear_Regression")
 #' install_course_github("jtleek", "Twitter_Map", "geojson")
 #' }
+#' @family InstallCourses
 install_course_github <- function(github_username, course_name, 
                                   branch="master", multi=FALSE){
   
@@ -280,6 +330,7 @@ install_course_github <- function(github_username, course_name,
 #' 
 #' install_course_dropbox("https://www.dropbox.com/s/xttkmuvu7hh72vu/my_course.zip")
 #' }
+#' @family InstallCourses
 install_course_dropbox <- function(url, multi=FALSE){
   # Construct url to the zip file
   zip_url <- paste0(sub("www.dropbox", "dl.dropboxusercontent", url), "?dl=1")
@@ -297,6 +348,7 @@ install_course_dropbox <- function(url, multi=FALSE){
 #' 
 #' install_course_google_drive("https://drive.google.com/file/d/F3fveiu873hfjZZj/edit?usp=sharing")
 #' }
+#' @family InstallCourses
 install_course_google_drive <- function(url, multi=FALSE){
   # Construct url to the zip file
   zip_url <- sub("file/d/", "uc?export=download&id=", 
@@ -317,6 +369,7 @@ install_course_google_drive <- function(url, multi=FALSE){
 #' 
 #' install_course_url("http://www.biostat.jhsph.edu/~rpeng/File_Hash_Course.zip")
 #' }
+#' @family InstallCourses
 install_course_url <- function(url, multi=FALSE){
   # Send GET request
   response <- GET(url)
