@@ -53,6 +53,10 @@ swirl <- function(resume.class="default", ...){
   removeTaskCallback("mini")
   # e lives here, in the environment created when swirl() is run
   e <- new.env(globalenv())
+  # This dummy object of class resume.class "tricks" the S3 system
+  # into calling the proper resume method. We retain the "environment"
+  # class so that as.list(e) works.
+  class(e) <- c("environment", resume.class)
   # The callback also lives in the environment created when swirl()
   # is run and retains a reference to it. Because of this reference,
   # the environment which contains both e and cb() persists as
@@ -63,9 +67,9 @@ swirl <- function(resume.class="default", ...){
     e$val <- val
     e$ok <- ok
     e$vis <- vis
-    # This dummy object of class resume.class "tricks" the S3 system
-    # into calling the proper resume method.
-    return(resume(structure(e,class=resume.class), ...))
+    # The result of resume() will determine whether the callback
+    # remains active
+    return(resume(e, ...))
   }
   addTaskCallback(cb, name="mini")
   invisible()
@@ -141,6 +145,31 @@ nxt <- function(){invisible()}
 #' }
 skip <- function(){invisible()}
 
+#' Start over on the current script question.
+#' 
+#' During a script question, this will reset the script
+#' back to its original state, which can be helpful if you
+#' get stuck.
+#' @export
+reset <- function(){invisible()}
+
+#' Submit the active R script in response to a question.
+#' 
+#' When a swirl question requires the user to edit an R script, the
+#' \code{submit()} function allows the user to submit their response.
+#' @export
+#' @examples
+#' \dontrun{
+#' 
+#' | Create a function called f that takes one argument, x, and
+#' | returns the value of x squared.
+#' 
+#' > submit()
+#' 
+#' | You are quite good my friend!
+#' }
+submit <- function(){invisible()}
+
 #' Tell swirl to ignore console input for a while.
 #' 
 #' It is sometimes useful to play around in the R console out of
@@ -211,19 +240,12 @@ main <- function(){invisible()}
 #' }
 info <- function(){
   swirl_out("When you are at the R prompt (>):")
-  
   swirl_out("-- Typing skip() allows you to skip the current question.", skip_before=FALSE)
-  
   swirl_out("-- Typing play() lets you experiment with R on your own; swirl will ignore what you do...", skip_before=FALSE)
   swirl_out("-- UNTIL you type nxt() which will regain swirl's attention.", skip_before=FALSE)
-  
   swirl_out("-- Typing bye() causes swirl to exit. Your progress will be saved.", skip_before=FALSE)
-  
   swirl_out("-- Typing main() returns you to swirl's main menu.", skip_before=FALSE)
-  
   swirl_out("-- Typing info() displays these options again.", skip_before=FALSE, skip_after=TRUE)
-
-  
   invisible()
 }
 
@@ -237,7 +259,7 @@ resume <- function(...)UseMethod("resume")
 # if necessary. The three instructions are themselves S3 methods which 
 # depend on the class of the active row of the course lesson. The 
 # instruction set is thus extensible. It can be found in R/instructionSet.R. 
-# 
+
 resume.default <- function(e, ...){
   # Check that if running in test mode, all necessary args are specified
   if(is(e, "test")) {
@@ -251,6 +273,22 @@ resume.default <- function(e, ...){
       e$test_lesson <- targs$test_lesson
       e$test_course <- targs$test_course
     }
+    # Check that 'from' is less than 'to' if they are both provided
+    if(!is.null(targs$from) && !is.null(targs$to)) {
+      if(targs$from >= targs$to) {
+        stop("Argument 'to' must be strictly greater than argument 'from'!")
+      }
+    }
+    if(is.null(targs$from)) {
+      e$test_from <- 1
+    } else {
+      e$test_from <- targs$from
+    }
+    if(is.null(targs$to)) {
+      e$test_to <- 999 # Lesson will end naturally before this
+    } else {
+      e$test_to <- targs$to
+    }
   }
   
   esc_flag <- TRUE
@@ -270,40 +308,89 @@ resume.default <- function(e, ...){
     e$playing <- FALSE
     e$iptr <- 1
   }
+  
+  # The user wants to reset their script to the original
+  if(uses_func("reset")(e$expr)[[1]]) {
+    e$playing <- FALSE
+    e$reset <- TRUE
+    e$iptr <- 2
+    swirl_out("I just reset the script to its original state. If it doesn't refresh immediately, you may need to click on it.", 
+              skip_after = TRUE)
+  }
+  
+  # The user wants to submit their R script
+  if(uses_func("submit")(e$expr)[[1]]){
+    e$playing <- FALSE
+    # Get contents from user's submitted script
+    e$script_contents <- readLines(e$script_temp_path, warn = FALSE)
+    # Save expr to e
+    e$expr <- try(parse(text = e$script_contents), silent = TRUE)
+    swirl_out("Sourcing your script...", skip_after = TRUE)
+    try(source(e$script_temp_path))
+  }
+  
   if(uses_func("play")(e$expr)[[1]]){
     swirl_out("Entering play mode. Experiment as you please, then type nxt() when you are ready to resume the lesson.", skip_after=TRUE)
     e$playing <- TRUE
   }
-  # If the user is playing, ignore console input,
-  # but remain in operation.
-  if(exists("playing", envir=e, inherits=FALSE) && e$playing){
-    esc_flag <- FALSE
-    return(TRUE)
-  }
+  
   # If the user wants to skip the current question, do the bookkeeping.
   if(uses_func("skip")(e$expr)[[1]]){
     # Increment a skip count kept in e.
-    if(!exists("skips", e))e$skips <- 0
+    if(!exists("skips", e)) e$skips <- 0
     e$skips <- 1 + e$skips
     # Enter the correct answer for the user
     # by simulating what the user should have done
-    #
     correctAns <- e$current.row[,"CorrectAnswer"]
-    # In case correctAns refers to newVar, add it
-    # to the official list AND the global environment
-    if(exists("newVarName",e)){
-      correctAns <- gsub("newVar", e$newVarName, correctAns)
+    
+    # If we are on a script question, the correct answer should
+    # simply source the correct script
+    if(is(e$current.row, "script") && is.na(correctAns)) {
+      correct_script_path <- e$correct_script_temp_path
+      if(file.exists(correct_script_path)) {
+        # Get contents of the correct script
+        e$script_contents <- readLines(correct_script_path, warn = FALSE)
+        # Save expr to e
+        e$expr <- try(parse(text = e$script_contents), silent = TRUE)
+        # Source the correct script
+        try(source(correct_script_path))
+        # Inform the user and open the correct script
+        swirl_out("I just sourced the following script, which demonstrates one possible solution.",
+                  skip_after=TRUE)
+        file.edit(correct_script_path)
+        readline("Press Enter when you are ready to continue...")
+      }
+      
+    # If this is not a script question...
+    } else {
+      # In case correctAns refers to newVar, add it
+      # to the official list AND the global environment
+      if(exists("newVarName",e)) {
+        correctAns <- gsub("newVar", e$newVarName, correctAns)
+      }
+      e$expr <- parse(text=correctAns)[[1]]
+      ce <- cleanEnv(e$snapshot)
+      e$val <- suppressMessages(suppressWarnings(eval(e$expr, ce)))
+      xfer(ce, globalenv())
+      ce <- as.list(ce)
+      
+      # Inform the user and expose the correct answer
+      swirl_out("Entering the following correct answer for you...",
+                skip_after=TRUE)
+      message("> ", e$current.row[, "CorrectAnswer"])
+      
     }
-    e$expr <- parse(text=correctAns)[[1]]
-    ce <- cleanEnv(e$snapshot)
-    e$val <- suppressMessages(suppressWarnings(eval(e$expr, ce)))
-    xfer(ce, globalenv())
-    ce <- as.list(ce)
-    # Inform the user, but don't expose the actual answer.    
-    swirl_out("Entering the following correct answer for you...",
-              skip_after=TRUE)
-    message("> ", e$current.row[, "CorrectAnswer"])
-  }
+    
+    # Make sure playing flag is off since user skipped
+    e$playing <- FALSE
+    
+  # If the user is not trying to skip and is playing, 
+  # ignore console input, but remain in operation.
+  } else if(exists("playing", envir=e, inherits=FALSE) && e$playing) {
+    esc_flag <- FALSE
+    return(TRUE)
+  }    
+  
   # If the user want to return to the main menu, do the bookkeeping
   if(uses_func("main")(e$expr)[[1]]){
     swirl_out("Returning to the main menu...")
@@ -312,6 +399,24 @@ resume.default <- function(e, ...){
       rm("les", envir=e, inherits=FALSE)
     }
   }
+  
+  # If user is looking up a help file, ignore their input
+  # unless the correct answer involves do so
+  if(uses_func("help")(e$expr)[[1]] || 
+       uses_func("`?`")(e$expr)[[1]]){
+    # Get current correct answer
+    corrans <- e$current.row[, "CorrectAnswer"]
+    # Parse the correct answer
+    corrans_parsed <- parse(text = corrans)
+    # See if it contains ? or help
+    uses_help <- uses_func("help")(corrans_parsed)[[1]] ||
+      uses_func("`?`")(corrans_parsed)[[1]]
+    if(!uses_help) {
+      esc_flag <- FALSE
+      return(TRUE)
+    }
+  }
+  
   # Method menu initializes or reinitializes e if necessary.
   temp <- mainMenu(e)
   # If menu returns FALSE, the user wants to exit.
@@ -329,15 +434,17 @@ resume.default <- function(e, ...){
   # Currently it can be set in customTests.R
   if(!uses_func("swirl")(e$expr)[[1]] &&
        !uses_func("swirlify")(e$expr)[[1]] &&
+       !uses_func("testit")(e$expr)[[1]] &&
        !uses_func("nxt")(e$expr)[[1]] &&
-       customTests$AUTO_DETECT_NEWVAR){
-    e$delta <- mergeLists(e$delta, safeEval(e$expr, e))
+       isTRUE(customTests$AUTO_DETECT_NEWVAR)) {
+    e$delta <- mergeLists(safeEval(e$expr, e), e$delta)
   }
   # Execute instructions until a return to the prompt is necessary
   while(!e$prompt){
     # If the lesson is complete, save progress, remove the current
     # lesson from e, and invoke the top level menu method.
-    if(e$row > nrow(e$les)){
+    # Below, min() ignores e$test_to if it is NULL (i.e. not in 'test' mode)
+    if(e$row > min(nrow(e$les), e$test_to)) {
       # If in test mode, we don't want to run another lesson
       if(is(e, "test")) {
         swirl_out("Lesson complete! Exiting swirl now...",
@@ -373,7 +480,7 @@ resume.default <- function(e, ...){
         swirl_out("Leaving swirl now. Type swirl() to resume.", skip_after=TRUE)
         esc_flag <- FALSE # to supress double notification
         return(FALSE)
-    }
+      }
     }
     # If we are ready for a new row, prepare it
     if(e$iptr == 1){      
